import textwrap


class TextBox:
    def __init__(self, text, width):
        self.text = text
        self.width = width
        self.string_list = self._generate_list()

    def __getitem__(self, item):
        return self.string_list[item]

    def _generate_list(self):
        # Split the input text into separate paragraphs before formatting the
        # length.
        paragraph_list = self.text.split("\n")
        text_list = []
        for paragraph in paragraph_list:
            text_list += textwrap.fill(paragraph, self.width, replace_whitespace=False).split("\n")
        text_list = [line.ljust(self.width) for line in text_list]
        return text_list

    def update_width(self, width):
        self.width = width
        self.string_list = self._generate_list()



import subprocess
import terminal_banner as tb


class Banner:
    kw_defaults = {
        'border_char': '*',
        'max_width': None,
        'min_width': None
    }

    def __init__(self, text, **kwargs):
        for key in Banner.kw_defaults:
            setattr(self, key, kwargs[key] if key in kwargs else self.kw_defaults[key])
        self.text = text
        self.width = None
        self.text_box = None

    def __str__(self):
        self._reset()
        return self._banner_string()

    def _banner_string(self):
        output_string = self.border_char * self.width + "\n"
        for text_line in self.text_box:
            output_string += self.border_char + " " + text_line + " " + self.border_char + "\n"
        output_string += self.border_char * self.width
        return output_string

    def _reset(self):
        self.width = self._calculate_width()
        self.text_box = tb.TextBox(self.text, self.width - 4)

    def _calculate_width(self):
        width = get_terminal_width()

        if self.max_width:
            width = min(width, self.max_width)

        if self.min_width:
            width = max(width, self.min_width)

        return width


def get_terminal_width():
    return int(subprocess.check_output(['stty', 'size']).split()[1])



from setuptools import setup
import re


VERSION_FILE = "terminal_banner/_version.py"
version_text = open(VERSION_FILE, "rt").read()
VERSION_REGEX = r"^__version__ = ['\"]([^'\"]*)['\"]"
version_match = re.search(VERSION_REGEX, version_text, re.M)
if version_match:
    version_string = version_match.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSION_FILE,))

setup(
    name='terminal_banner',
    version=version_string,
    description='Text Banner for Terminals',
    url='https://github.com/martincyoung/terminal_banner',
    packages=['terminal_banner'],
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
    ],
    author='Martin Young',
    author_email='martin.young@cantab.net'
)
